import SDL;

import SDL_gfxPrimitives;

import mysdl;
import log;

import std.string;
import std.random;

alias std.random random;

class Game {
public:
	this(SDL_Surface* scr) {
		Log.info("game init");

		scr_ = scr;

		map_ = MySDL.createRGBSurface(SDL_SWSURFACE, 640, 480, 8);
		map_ = MySDL.displayFormat(map_);
		if (map_ == (SDL_Surface*)0) {
			Log.errorSDL("memory too low?");
			exit(1);
		}

		for (int i = 0; i < BLOCKS; i++) {
			filledCircleRGBA(map_, random.rand()%640, random.rand()%480, 16,
							 255,255,255,255);
		}
		x1 = tx1 = random.rand()%640;
		y1 = ty1 = random.rand()%480;
		x2 = tx2 = random.rand()%640;
		y2 = ty2 = random.rand()%480;

		vx1 = vx2 = vy1 = vy2 = 0;
		w1 = w2 = 0;

		filledCircleRGBA(map_, tx1, ty1, 16, 255, 0, 0, 255);
		filledCircleRGBA(map_, tx2, ty2, 16, 0, 0, 255, 255);

		end_ = false;
		quit_ = false;

		first = SDL_GetTicks();
	}

	~this() {
		SDL_FreeSurface(map_);
	}

	void run() {
		Log.info("running...");

		while (!end_) {
			runClear();
			runMove();
			runDraw();
			runEndCheck();
			runWait();
		}

		if (!quit_) {
			SDL_BlitSurface(map_, (SDL_Rect*)0, scr_, (SDL_Rect*)0);
			SDL_UpdateRect(scr_, 0,0,0,0);

			SDL_Delay(2000);

			SDL_Event ev;

			while (SDL_PollEvent(&ev)) {}

			bit wait = true;
			while (wait) {
				while (SDL_PollEvent(&ev)) {
					if (ev.type == SDL_QUIT) {
						end_ = quit_ = true;
					}
					else if (ev.type == SDL_KEYDOWN) {
						wait = false;
					}
				}
			}
		}
	}

	void runClear() {
		SDL_FillRect(scr_, (SDL_Rect*)0, SDL_MapRGB(scr_.format, 0,0,0));
	}

	void runMove() {
		SDL_PumpEvents();

		Uint8* keys = SDL_GetKeyState((int*)0);

		if (w1 == 0) {
			if (keys[SDLK_UP]) {
				vy1 = -1;
				if (keys[SDLK_RIGHT]) vx1 = 1;
				else if (keys[SDLK_LEFT]) vx1 = -1;
				else vx1 = 0;
			}
			else if (keys[SDLK_DOWN]) {
				vy1 = 1;
				if (keys[SDLK_RIGHT]) vx1 = 1;
				else if (keys[SDLK_LEFT]) vx1 = -1;
				else vx1 = 0;
			}
			else {
				if (keys[SDLK_RIGHT]) {
					vx1 = 1;
					vy1 = 0;
				}
				else if (keys[SDLK_LEFT]) {
					vx1 = -1;
					vy1 = 0;
				}
			}
			x1 += vx1;
			y1 += vy1;

			Uint8* p = (Uint8*)map_.pixels+y1*map_.pitch+x1;
			if (x1 < 0 || x1 > 639 || y1 < 0 || y1 > 479 ||
				*p == SDL_MapRGB(map_.format, 255,255,255))
			{
				filledCircleRGBA(map_, x1, y1, 16,
								 0,0,0,255);
				w1 = 30;
			}
			else if ((x1-tx2)*(x1-tx2)+(y1-ty2)*(y1-ty2) < 256) {
				filledCircleRGBA(map_, x1, y1, 16,
								 0,0,0,255);
				end_ = true;
			}
		}
		else {
			w1--;
			if (w1 == 0) {
				x1 = tx1;
				y1 = ty1;
				vx1 = vy1 = 0;
			}
		}

		if (w2 == 0) {
			if (keys['w']) {
				vy2 = -1;
				if (keys['d']) vx2 = 1;
				else if (keys['a']) vx2 = -1;
				else vx2 = 0;
			}
			else if (keys['s']) {
				vy2 = 1;
				if (keys['d']) vx2 = 1;
				else if (keys['a']) vx2 = -1;
				else vx2 = 0;
			}
			else {
				if (keys['d']) {
					vx2 = 1;
					vy2 = 0;
				}
				else if (keys['a']) {
					vx2 = -1;
					vy2 = 0;
				}
			}
			x2 += vx2;
			y2 += vy2;

			Uint8* p = (Uint8*)map_.pixels+y2*map_.pitch+x2;
			if (x2 < 0 || x2 > 639 || y2 < 0 || y2 > 479 ||
				*p == SDL_MapRGB(map_.format, 255,255,255))
			{
				filledCircleRGBA(map_, x2, y2, 16,
								 0,0,0,255);
				w2 = 30;
			}
			else if ((x2-tx1)*(x2-tx1)+(y2-ty1)*(y2-ty1) < 256) {
				filledCircleRGBA(map_, x1, y1, 16,
								 0,0,0,255);
				end_ = true;
			}
		}
		else {
			w2--;
			if (w2 == 0) {
				x2 = tx2;
				y2 = ty2;
				vx2 = vy2 = 0;
			}
		}

		pixelRGBA(map_, x1, y1, 255,0,0,255);
		pixelRGBA(map_, x2, y2, 0,0,255,255);
	}

	void runWait() {
		SDL_UpdateRect(scr_, 0,0,0,0);

		turn++;
		int ellapsed = SDL_GetTicks() - first;
		if (turn * 16 > ellapsed) {
			SDL_Delay(turn * 16 - ellapsed);
		}
	}

	void runDraw() {
		SDL_Rect sr, dr;
		sr.w = sr.h = dr.w = dr.h = 81;

		sr.x = x1 - 40;
		sr.y = y1 - 40;
		dr.x = 100;
		dr.y = 300;
		SDL_FillRect(scr_, &dr, SDL_MapRGB(scr_.format, 255,255,255));
		SDL_BlitSurface(map_, &sr, scr_, &dr);
		sr.x = x2 - 40;
		sr.y = y2 - 40;
		dr.x = 640-140;
		dr.y = 300;
		dr.w = dr.h = 81;
		SDL_FillRect(scr_, &dr, SDL_MapRGB(scr_.format, 255,255,255));
		SDL_BlitSurface(map_, &sr, scr_, &dr);
	}

	void runEndCheck() {
        SDL_Event ev;
        while (SDL_PollEvent(&ev)) {
			if (ev.type == SDL_QUIT) {
				end_ = quit_ = true;
			}
        }
	}

	bit isEnd() {
		return quit_;
	}

private:
	SDL_Surface* scr_;
	SDL_Surface* map_;

	Sint16 x1, y1, x2, y2;
	Sint16 tx1, ty1, tx2, ty2;
	Sint16 vx1, vy1, vx2, vy2;
	int w1, w2;

	int turn;
	Uint32 first;

	final int BLOCKS = 256;

	bit end_, quit_;
}
